//******************************************************************************
//  MSP-FET430P410 Demo - Timer_A UART 9600 Echo, DCO SMCLK 
//
//  Description: Use timer_A CCR0 hardware output modes and SCCI data latch to
//  to implement UART function @ 9600 baud. Software does not directly read and
//  write to RX and TX pins, instead proper use of output modes and SCCI data
//  latch are demonstrated. Use of these hardware features eliminates ISR 
//  latency effects as hardware insures that output and input bit latching and 
//  timing are perfectly synchronized with timer_A regardless of other 
//  software activity. In the Mainloop the UART function readies the UART to 
//  receive one character and waits in LPM0 with all activity interrupt driven. 
//  After a character has been received, the UART receive function forces exit 
//  from LPM0 in the Mainloop which echo's back the received character.
//  ACLK = LFXT1 = 32768, MCLK = SMCLK = TACLK = DCO = 32xACLK = 1048576  
//  //*An external watch crystal is required on XIN XOUT for ACLK*//	  
//
//                MSP430F413
//            -----------------
//        /|\|              XIN|-        
//         | |                 | 32kHz     
//         --|RST          XOUT|-        
//           |                 |         
//           |    CCI0B/RX/P1.1|<--------  
//           |                 | 9600 8N1 
//           |    CCI0A/TX/P1.0|-------->




//  M.Buccini
//  Texas Instruments, Inc
//  March 2002
//******************************************************************************

#include "uart.h"

void init_uart( void ) 
{
	P1SEL = TXD + RXD;				// PP1.0/1 TA0 for TXD/RXD function.
									// peripheral module function is 
									// selected for the pin TXD + RXD
  	P1DIR = TXD;					// TXD output on P1
}

// Transmit the chacter c 

void send_char(int c)
{
  RXTXData = c;
  BitCnt = 0xA;                         // Load Bit counter, 8data + ST/SP
  CCR0 = TAR;                           // Current state of TA counter
  CCR0 += Bitime;                       // Some time till first bit
  RXTXData |= 0x100;                    // Add mark stop bit to RXTXData 
  RXTXData = RXTXData << 1;             // Add space start bit
  CCTL0 = OUTMOD0+CCIE;                 // TXD = mark = idle 
  while ( CCTL0 & CCIE );               // Wait for TX completion
	
}

// Function Transmits Character from RXTXData Buffer
//void TX_Byte (void)
//{
//  BitCnt = 0xA;                         // Load Bit counter, 8data + ST/SP
//  CCR0 = TAR;                           // Current state of TA counter
//  CCR0 += Bitime;                       // Some time till first bit
//  RXTXData |= 0x100;                    // Add mark stop bit to RXTXData 
//  RXTXData = RXTXData << 1;             // Add space start bit
//  CCTL0 = OUTMOD0+CCIE;                 // TXD = mark = idle 
//  while ( CCTL0 & CCIE );               // Wait for TX completion
//}


// Function Readies UART to Receive Character into RXTXData Buffer
//void RX_Ready (void)
//{
//  BitCnt = 0x8;                         // Load Bit counter
//  CCTL0 = SCS+CCIS0+OUTMOD_SET+CM_NEG+CAP+CCIE;   // Sync, Input P1.0/TA0, Neg Edge, Capture 
//  _BIS_SR(CPUOFF+GIE);                  // Enter LPM0 Until character RXed         
//}

int recv_char(void)
{
  BitCnt = 0x8;                         // Load Bit counter
  CCTL0 = SCS+CCIS0+OUTMOD_SET+CM_NEG+CAP+CCIE;   // Sync, Input P1.0/TA0, Neg Edge, Capture 
  _BIS_SR(CPUOFF+GIE);                  // Enter LPM0 Until character RXed 
  return RXTXData;
  
}
